package com.bdifn.hbase.hfile;

import java.io.ByteArrayInputStream;
import java.net.URI;
import java.nio.ByteBuffer;
import java.util.Arrays;
import java.util.List;

import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FSDataInputStream;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.hbase.KeyValue;
import org.apache.hadoop.hbase.io.hfile.FixedFileTrailer;
import org.apache.hadoop.hbase.protobuf.ProtobufUtil;

import com.bdifn.hbase.hfile.proto.FileInfoProtos;
import com.bdifn.hbase.hfile.proto.FileInfoProtos.BytesBytesPair;
import com.bdifn.hbase.hfile.proto.HFileProtos.FileTrailerProto;

public class HFileDemo4 {

	/**
	 * @param args
	 */
	public static void main(String[] args) throws Exception {
		Configuration config = new Configuration();

		// Path path = new Path("/test/0a99d83b2b0a49c0adbc371d4bfe021e");
		String pathStr = "file:///f:/0a99d83b2b0a49c0adbc371d4bfe021e";
		Path path = new Path(pathStr);

		FileSystem fs = FileSystem.get(URI.create(pathStr), config);

		FSDataInputStream input = fs.open(path);

		long length = input.available();
		input.seek(length - 1);

		System.out.println(length);
		final FixedFileTrailer trailer = FixedFileTrailer.readFromStream(input,
				length);

		System.out.println(trailer);

		int trailerSize = 212;

		input.seek(length - trailerSize);

		byte[] trailerBytes = new byte[212];

		int readLen = input.read(trailerBytes);
		System.out.println(readLen);
		// print magic code
		System.out.println(new String(Arrays.copyOfRange(trailerBytes, 0, 8)));
		// System.out.println(String.format("MemStoreSize:%d\n,HFileSize:%d\n,KeyValue Count:%d",
		// length,reader.length(),count));

		// FileTrailerProto hfileProtos =
		// FileTrailerProto.newBuilder().mergeFrom(Arrays.copyOfRange(trailerBytes,
		// 8, trailerBytes.length -4 - 8)).build();

		FileTrailerProto hfileProtos = FileTrailerProto.PARSER
				.parseDelimitedFrom(new ByteArrayInputStream(trailerBytes, 8,
						trailerBytes.length - 4));

		System.out.println(hfileProtos);

		long loadOnOpenLength = length - trailerSize
				- hfileProtos.getLoadOnOpenDataOffset();

		MyHFileBlockIterator inter = new MyHFileBlockIterator(input,
				hfileProtos.getLoadOnOpenDataOffset(), (int) loadOnOpenLength);

		MyHFileBlock dataIndex = inter.nextBlock();
		System.out.println(dataIndex.getHeader());

		int dataIndexLevels = hfileProtos.getNumDataIndexLevels();
		int dataIndexEntries = hfileProtos.getDataIndexCount();

		MyHFileBlock.BlockIndexReader rootDataReader = dataIndex
				.createBlockIndexReader().parseMultiLevel(dataIndexEntries,
						"IDXROOT2", dataIndexLevels);

		MyHFileBlock metaIndex = inter.nextBlock();

		System.out.println(metaIndex.getHeader());

		MyHFileBlock fileInfo = inter.nextBlock();

		System.out.println(fileInfo.getHeader() + ":"
				+ fileInfo.getBlockBuf().capacity());

		ByteArrayInputStream in = new ByteArrayInputStream(fileInfo
				.getBlockBuf().array());
		int pblen = ProtobufUtil.lengthOfPBMagic();

		byte[] pbuf = new byte[pblen];
		if (in.markSupported())
			in.mark(pblen);
		int read = in.read(pbuf);

		FileInfoProtos.FileInfoProto fileInfoProto = FileInfoProtos.FileInfoProto
				.parseDelimitedFrom(in);

		List<BytesBytesPair> entries = fileInfoProto.getMapEntryList();

		for (BytesBytesPair entry : entries) {
			System.out.println(entry.getFirst().toStringUtf8() + ":"
					+ entry.getSecond().toStringUtf8());
		}

		System.out.println(pblen + ":" + inter.hasNext());

		while (inter.hasNext()) {
			MyHFileBlock block = inter.nextBlock();
			System.out.println(block.getHeader());
		}

		// load datablock
		long offset = rootDataReader.getBlockOffsets()[0];
		int size = rootDataReader.getBlockDataSizes()[0];

		byte[] dataBlockArray = new byte[size];
		input.seek(offset);
		input.read(dataBlockArray);

		MyHFileBlockIterator dataBlockIter = new MyHFileBlockIterator(
				dataBlockArray);

		MyHFileBlock dataBlock1 = dataBlockIter.nextBlock();

		System.out.println(dataBlock1.getHeader());

		KeyValueIterator kvIter = new KeyValueIterator(dataBlock1.getBlockBuf()
				.array());

		int kvCount = 0;
		while (kvIter.hasNext()) {
			KeyValue kv = kvIter.nextKeyValue();
			System.out.println(kv);
			kvCount ++;
		}
		
		System.out.println("kvCount:" + kvCount);
	}

}
