package com.bdifn.hbase.hfile;

import org.apache.hadoop.hbase.KeyValue;
import org.apache.hadoop.hbase.util.Bytes;

public class KeyValueIterator {
    
    public static final int KEY_LENGTH_SIZE = 4;
    public static final int VALUE_LENGTH_SIZE = 4;
    
    private byte [] data ;
    private int currentOffset ;
    
    public KeyValueIterator(byte [] data) {
        this.data = data;
        currentOffset = 0;
    }
    
    
    public KeyValue nextKeyValue(){
        KeyValue kv = null;
        
        int keyLen = Bytes.toInt(data,currentOffset,4);
        incrementOffset(KEY_LENGTH_SIZE);
        
        int valueLen = Bytes.toInt(data,currentOffset,4);
        incrementOffset(VALUE_LENGTH_SIZE);
        
//        decodeKeyValueManually(keyLen,valueLen);
        
        //1 is memTS
        incrementOffset(keyLen,valueLen,1);
        
        int kvSize = KEY_LENGTH_SIZE + VALUE_LENGTH_SIZE +  keyLen + valueLen ;

        kv = new KeyValue(data , currentOffset - kvSize - 1, kvSize);
      
        return kv;
    }
    
    public void decodeKeyValueManually(int keyLen,int valueLen) {

        short rowKeyLen = Bytes.toShort(data,currentOffset ,2 );
        int rowKey = Bytes.toInt(data,currentOffset + 2,rowKeyLen);
        
        byte cfLen = data[currentOffset + 2 + rowKeyLen ];
        String columnFamily = Bytes.toString( data ,currentOffset + 2 + rowKeyLen + 1,  cfLen);
        
        int qualifierLen = keyLen - 8 - rowKeyLen - 1 - cfLen - 1;
        
        String qualifier = Bytes.toString( data ,currentOffset + 2 + rowKeyLen + 1 + cfLen ,  qualifierLen);
        
        int tsOffset = currentOffset + keyLen - 9;
        
        long ts =  Bytes.toLong(data,tsOffset);
        
        byte type = data[ currentOffset + keyLen - 1 ];
        
        
        String value = Bytes.toString( data , currentOffset,  valueLen);
        
        System.out.println( rowKey +":" + columnFamily + ":" + qualifier + ":" + ts + ":" + type);
    }
    
    
    
    public void incrementOffset(int ... lengths) {
        for(int length : lengths)
            currentOffset = currentOffset + length;
    }
    
    public boolean hasNext() {
        return currentOffset < data.length;
    }
}
