package com.bdifn.hbase.hfile;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.nio.ByteBuffer;

import org.apache.hadoop.hbase.util.Bytes;

public class MyHFileBlock {

    public static final int HARDER_SIZE = 33;

    public static class Header {
        private String magicCode;
        int onDiskSizeWithoutHeader;
        int unCompressBlockSize;
        long prevBlockOffset;
        byte checkSum;
        int bytesPerChecksum;
        int onDiskDataSizeWithHeader;

        public String getMagicCode() {
            return magicCode;
        }

        public void setMagicCode(String magicCode) {
            this.magicCode = magicCode;
        }

        public int getOnDiskSizeWithoutHeader() {
            return onDiskSizeWithoutHeader;
        }

        public void setOnDiskSizeWithoutHeader(int onDiskSizeWithoutHeader) {
            this.onDiskSizeWithoutHeader = onDiskSizeWithoutHeader;
        }

        public int getUnCompressBlockSize() {
            return unCompressBlockSize;
        }

        public void setUnCompressBlockSize(int unCompressBlockSize) {
            this.unCompressBlockSize = unCompressBlockSize;
        }

        public long getPrevBlockOffset() {
            return prevBlockOffset;
        }

        public void setPrevBlockOffset(long prevBlockOffset) {
            this.prevBlockOffset = prevBlockOffset;
        }

        public byte getCheckSum() {
            return checkSum;
        }

        public void setCheckSum(byte checkSum) {
            this.checkSum = checkSum;
        }

        public int getBytesPerChecksum() {
            return bytesPerChecksum;
        }

        public void setBytesPerChecksum(int bytesPerChecksum) {
            this.bytesPerChecksum = bytesPerChecksum;
        }

        public int getOnDiskDataSizeWithHeader() {
            return onDiskDataSizeWithHeader;
        }

        public void setOnDiskDataSizeWithHeader(int onDiskDataSizeWithHeader) {
            this.onDiskDataSizeWithHeader = onDiskDataSizeWithHeader;
        }

        public Header() {
            super();
        }

        public void initHeader(ByteBuffer buf) {
            byte[] magicCodeArray = new byte[8];
            buf.get(magicCodeArray, 0, magicCodeArray.length);

            magicCode = new String(magicCodeArray);

            onDiskSizeWithoutHeader = buf.getInt();
            unCompressBlockSize = buf.getInt();
            prevBlockOffset = buf.getLong();
            checkSum = buf.get();
            bytesPerChecksum = buf.getInt();
            onDiskDataSizeWithHeader = buf.getInt();
        }

        public Header(String magicCode, int onDiskSizeWithoutHeader, int unCompressBlockSize, long prevBlockOffset,
                byte checkSum, int bytesPerChecksum, int onDiskDataSizeWithHeader) {
            super();
            this.magicCode = magicCode;
            this.onDiskSizeWithoutHeader = onDiskSizeWithoutHeader;
            this.unCompressBlockSize = unCompressBlockSize;
            this.prevBlockOffset = prevBlockOffset;
            this.checkSum = checkSum;
            this.bytesPerChecksum = bytesPerChecksum;
            this.onDiskDataSizeWithHeader = onDiskDataSizeWithHeader;
        }

        @Override
        public String toString() {
            return "Header [magicCode=" + magicCode + ", onDiskSizeWithoutHeader=" + onDiskSizeWithoutHeader
                    + ", unCompressBlockSize=" + unCompressBlockSize + ", prevBlockOffset=" + prevBlockOffset
                    + ", checkSum=" + checkSum + ", bytesPerChecksum=" + bytesPerChecksum
                    + ", onDiskDataSizeWithHeader=" + onDiskDataSizeWithHeader + "]";
        }

    }

    private Header header;
    private ByteBuffer blockBuf;
    private byte[] checkSum;

    public MyHFileBlock(ByteBuffer buf) {
        header = new Header();
        header.initHeader(buf);
    }
    
    public BlockIndexReader createBlockIndexReader(){
       return  new BlockIndexReader(this);
    }

    public static class BlockIndexReader {
        // Root-level data.
        private byte[][] blockKeys;
        private long[] blockOffsets;
        private int[] blockDataSizes;
        private int rootCount = 0;

        // Mid-key metadata.
        private long midLeafBlockOffset = -1;
        private int midLeafBlockOnDiskSize = -1;
        private int midKeyEntry = -1;

        private int level;
        
        private DataInputStream dataInput;
        private MyHFileBlock block;

        public BlockIndexReader(MyHFileBlock block) {
            dataInput = new DataInputStream(new ByteArrayInputStream(block.getBlockBuf().array()));
            this.block = block;
        }

        public BlockIndexReader parseMultiLevel(int numEntries, String expectedMagicCode, int level) throws Exception {

            parse(numEntries, expectedMagicCode);
            this.level= level;
            
            if (dataInput.available() < 16) {
                // No mid-key metadata available.
                return this;
            }

            midLeafBlockOffset = dataInput.readLong();
            midLeafBlockOnDiskSize = dataInput.readInt();
            midKeyEntry = dataInput.readInt();
            return this;
        }

        public BlockIndexReader parse(int numEntries, String expectedMagicCode) throws Exception {
            if (!block.getHeader().getMagicCode().equals(expectedMagicCode))
                throw new RuntimeException(String.format("epect Magic code is %s ,but actual is %s", expectedMagicCode,
                        block.getHeader().getMagicCode()));

            blockOffsets = new long[numEntries];
            blockKeys = new byte[numEntries][];
            blockDataSizes = new int[numEntries];
            
            level = 1; 
            for (int i = 0; i < numEntries; i++) {
                long offset = dataInput.readLong();
                int dataSize = dataInput.readInt();
                byte[] key = Bytes.readByteArray(dataInput);

                blockOffsets[i] = offset;
                blockDataSizes[i] = dataSize;
                blockKeys[i] = key;
                
                rootCount ++;
            }
            return this;
        }

        public byte[][] getBlockKeys() {
            return blockKeys;
        }

        public void setBlockKeys(byte[][] blockKeys) {
            this.blockKeys = blockKeys;
        }

        public long[] getBlockOffsets() {
            return blockOffsets;
        }

        public void setBlockOffsets(long[] blockOffsets) {
            this.blockOffsets = blockOffsets;
        }

        public int[] getBlockDataSizes() {
            return blockDataSizes;
        }

        public void setBlockDataSizes(int[] blockDataSizes) {
            this.blockDataSizes = blockDataSizes;
        }

        public int getRootCount() {
            return rootCount;
        }

        public void setRootCount(int rootCount) {
            this.rootCount = rootCount;
        }

        public long getMidLeafBlockOffset() {
            return midLeafBlockOffset;
        }

        public void setMidLeafBlockOffset(long midLeafBlockOffset) {
            this.midLeafBlockOffset = midLeafBlockOffset;
        }

        public int getMidLeafBlockOnDiskSize() {
            return midLeafBlockOnDiskSize;
        }

        public void setMidLeafBlockOnDiskSize(int midLeafBlockOnDiskSize) {
            this.midLeafBlockOnDiskSize = midLeafBlockOnDiskSize;
        }

        public int getMidKeyEntry() {
            return midKeyEntry;
        }

        public void setMidKeyEntry(int midKeyEntry) {
            this.midKeyEntry = midKeyEntry;
        }

        public int getLevel() {
            return level;
        }

        public void setLevel(int level) {
            this.level = level;
        }

    }

    public Header getHeader() {
        return header;
    }

    public void setHeader(Header header) {
        this.header = header;
    }

    public byte[] getCheckSum() {
        return checkSum;
    }

    public void setCheckSum(byte[] checkSum) {
        this.checkSum = checkSum;
    }

    public ByteBuffer getBlockBuf() {
        return blockBuf;
    }

    public void setBlockBuf(ByteBuffer blockBuf) {
        this.blockBuf = blockBuf;
    }
}
