package com.bdifn.hbase.hfile;

import java.io.IOException;
import java.nio.ByteBuffer;

import org.apache.hadoop.fs.FSDataInputStream;

import com.bdifn.hbase.hfile.MyHFileBlock.Header;

public class MyHFileBlockIterator {

	private ByteBuffer loadOnOpenBuffer;

	public MyHFileBlockIterator(FSDataInputStream data, long offset, int length) {

		try {
			data.seek(offset);
			byte[] loadOnOpenBytes = new byte[length];
			data.read(loadOnOpenBytes);
			loadOnOpenBuffer = ByteBuffer.wrap(loadOnOpenBytes);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

    public MyHFileBlockIterator(byte [] data) {
        loadOnOpenBuffer = ByteBuffer.wrap(data);
    }
    
	public MyHFileBlock nextBlock() {

		MyHFileBlock block = new MyHFileBlock(loadOnOpenBuffer);
		Header header = block.getHeader();
		int currentBlockLength = block.getHeader()
				.getOnDiskDataSizeWithHeader();

		int dataSize = currentBlockLength - MyHFileBlock.HARDER_SIZE;

		byte[] dataBlockArray = new byte[dataSize];

		loadOnOpenBuffer.get(dataBlockArray);

		ByteBuffer dataBlock = ByteBuffer.wrap(dataBlockArray);
	 
		block.setBlockBuf(dataBlock);

		int checkSumChunks = header.getOnDiskSizeWithoutHeader()
				/ header.getBytesPerChecksum();
		if (header.getOnDiskSizeWithoutHeader() % header.getBytesPerChecksum() != 0) {
			checkSumChunks++;
		}
		int checkSumBytes = checkSumChunks * 4;
		byte[] checkSum = new byte[checkSumBytes];

		loadOnOpenBuffer.get(checkSum);

		block.setCheckSum(checkSum);

		return block;
	}
	
	public boolean hasNext(){
	    return loadOnOpenBuffer.position() < loadOnOpenBuffer.capacity();
	}
}
